"""Manag the files and data"""
import os
import sys
import json
from datetime import datetime
from wmc.default import COMMON, LINUX, WIN


class ProjectFiles():
    """docstring for ProjectFiles."""

    KEYS = ['path', 'data', 'videos', 'final', 'intro']

    def __init__(self, path):
        super(ProjectFiles, self).__init__()
        self.path = os.path.abspath(path)

    def __getitem__(self, key):
        """ ProjectFiles['info'] """
        if key == 'path':
            return self.path
        if key == 'data':
            return os.path.join(self.path, 'data.json')
        if key == 'videos':
            return list(self.videos())
        if key in ['final', 'intro']:
            return os.path.join(self.path, f'{key}.mp4')
        return None

    def check(self):
        """Check if the project is healthy."""
        return os.path.isfile(self['data'])

    def videos(self):
        """Iterator to grep the video files"""
        for name in os.listdir(self['path']):
            file = os.path.join(self['path'], name)
            if os.path.isfile(file) and name.endswith('.mp4'):
                yield file


class ProjectData():
    """docstring for ProjectData."""

    KEYS = ['name', 'record', 'size', 'prefix', 'intro', 'intro-record']

    def __init__(self, filename, profil=None):
        super(ProjectData, self).__init__()
        self.filename = filename
        self.profil = profil
        self.data = {}

    def __getitem__(self, key):
        """ ProjectData['info'] """
        if key in self.KEYS:
            return self.data.get(key)
        return None

    def __contains__(self, key):
        if key in self.KEYS:
            return True
        return False

    def load(self):
        """Load the data from the file"""
        with open(self.filename) as file:
            self.data = json.load(file)
        if self.profil and self.profil in self.data['profils']:
            self.data = self.data['profils'][self.profil]

    def save(self):
        """Save the data to the file"""
        with open(self.filename, 'w') as file:
            json.dump(self.data, file, indent=4, sort_keys=True)

    def create(self):
        """Create the data file"""
        self.data = {
            'name': os.path.basename(os.path.dirname(self.filename)),
            'profils': {}
        }
        for key, value in COMMON.items():
            self.data[key] = value

        if sys.platform in ['linux', 'linux2']:
            defaults = LINUX
        elif sys.platform in ['win32', 'win64']:
            defaults = WIN
        else:
            defaults = {}
        for key, value in defaults.items():
            self.data[key] = value

        self.save()

    def check(self):
        """Check if the data file is healthy."""
        for key in self.KEYS:
            if key not in self.data:
                return False
        return True


class Project():
    """Manage the data and files for one project folder."""

    def __init__(self, path, profil=None):
        super(Project, self).__init__()
        self.files = ProjectFiles(path)
        self.data = ProjectData(self.files['data'], profil)

    def __getitem__(self, key):
        """ ProjectFiles['info'] """
        if key == 'video':
            now = datetime.now().strftime('%Y%m%d%H%M.mp4')
            return os.path.join(self.files['path'], self.data['prefix'] + now)
        if key == 'records':
            return sorted(list(filter(lambda x: x.find(self.data['prefix']) > 0, self.files['videos'])))
        return None

    def __contains__(self, key):
        if key in ['video', 'records']:
            return True
        return False

    def create(self):
        """Create the info file"""
        if os.path.isfile(self.files['path']):
            raise Exception('Your project look like a file')

        if os.path.isdir(self.files['path']) and os.listdir(self.files['path']):
            raise Exception('The folder is not empty. '
                            'You have to delete it yourself.')

        if not os.path.isdir(self.files['path']):
            os.makedirs(self.files['path'])
        self.data.create()

    def check(self):
        """Check if the project is healthy."""
        return self.files.check() and self.data.check()



import unittest
from wmc.utils import resize_window


class TestResizeWindow(unittest.TestCase):

    def test_call(self):
        callable = hasattr(resize_window, '__call__')
        self.assertTrue(callable)


if __name__ == '__main__':
    unittest.main()



import tempfile
import shutil


class TempdirManager(object):

    def setUp(self):
        super().setUp()
        self.tempdirs = []

    def tearDown(self):
        super().tearDown()
        while self.tempdirs:
            shutil.rmtree(self.tempdirs.pop())

    def mkdtemp(self):
        self.tempdirs.append(tempfile.mkdtemp())
        return self.tempdirs[-1]



"""Some useful tools"""
import sys
from time import sleep


def resize_window(size, wait=1):
    """Resize the current window"""
    sleep(wait)
    if sys.platform in ["win32", "win64"]:
        import win32gui
        hwnd = win32gui.GetForegroundWindow()
        win32gui.MoveWindow(hwnd, *size, True)

    elif sys.platform in ["linux", "linux2"]:
        import Xlib
        import Xlib.display
        display = Xlib.display.Display()
        root = display.screen().root
        wid = root.get_full_property(
            display.intern_atom("_NET_ACTIVE_WINDOW"),
            Xlib.X.AnyPropertyType,
        ).value[0]
        window = display.create_resource_object("window", wid)
        window.configure(
            x=size[0], y=size[1], width=size[2], height=size[3]
        )
        display.sync()



"""Assemble the data and command together"""
import os
import logging
import ffmpeg
from lying.utils import Terminal
from wmc.project import Project
from wmc.utils import resize_window


class Interface():
    """docstring for Interface."""

    def __init__(self, path, profil=None, create=False, check=True):
        self.logger = logging.getLogger(self.__class__.__name__)
        self.proj = Project(path, profil)

        if create:
            self.proj.create()

        if os.path.isfile(self.proj.files['data']):
            self.proj.data.load()

        if check and not self.proj.check():
            raise Exception('This is not a wmc project.')

        self.logger.info('Create a Project object')

    def info(self):
        """Print some project infos"""
        print('Name:   ', self.proj['name'])
        print('Records:', len(self.proj['records']))

    def record(self):
        """start the record"""
        filename = self.proj['video']
        setting = self.proj.data['record']
        stream = ffmpeg.input(**setting["input"]).setpts(setting["setpts"])
        stream = ffmpeg.output(stream, filename, **setting["output"])
        ffmpeg.run(stream, overwrite_output=True)

    def size(self):
        """change the size of the foreground window"""
        resize_window(self.proj.data['size'])

    def link(self):
        """concat all videos to one"""
        stream = ffmpeg.input(self.proj['records'][0])
        for video in self.proj['records'][1:]:
            stream = stream.concat(ffmpeg.input(video))
        stream = ffmpeg.output(stream, self.proj.files['final'])
        ffmpeg.run(stream, overwrite_output=True)

    def intro(self):
        """create a nice intro"""
        setting = self.proj.data["intro-record"]
        stream = ffmpeg.input(**setting["input"])
        stream = ffmpeg.output(stream, self.proj.files['intro'], **setting["output"])
        process = ffmpeg.run_async(
            stream,
            pipe_stdin=True,
            pipe_stdout=True,
            pipe_stderr=True,
            overwrite_output=True,
        )

        cmds = self.proj.data["intro"]["cmds"]
        prompt = self.proj.data["intro"].get("prompt", ">> ")
        wait = self.proj.data["intro"].get("wait", 1)
        terminal = Terminal(cmds, prompt=prompt, wait=wait)
        terminal.run(auto_exit=True)

        process.communicate(input=b"q")



"""The cli"""
import os
import sys
import logging
from argparse import ArgumentParser
from wmc import __version__
from wmc.assemble import Interface


def main():
    """print some help"""
    parser = ArgumentParser(
        description='Watch me coding, a toolbox',
        epilog='Copyright 2019 AxJu | WMCv{}'.format(__version__),
    )
    parser.add_argument(
        'action', choices=('setup', 'info', 'record', 'size', 'link', 'intro'),
        help='Select the action'
    )
    parser.add_argument(
        'path', nargs='?', default=os.getcwd(),
        help='Path to the project'
    )
    parser.add_argument(
        '--verbose', action='store_true',
        help='enable debug infos'
    )
    parser.add_argument(
        '--version', action='store_true',
        help='Print program version and exit'
    )

    args = parser.parse_args()
    if args.version:
        print(__version__)
        sys.exit(0)
    if args.verbose:
        log_format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
        logging.basicConfig(level=logging.DEBUG, format=log_format)

    interface = Interface(args.path, create=args.action == 'setup')
    if args.action == 'setup':
        print('Create new project')

    if args.action in ['info', 'setup']:
        interface.info()

    if args.action == 'record':
        interface.record()

    if args.action == 'size':
        interface.size()

    if args.action == 'link':
        interface.link()

    if args.action == 'intro':
        interface.intro()


if __name__ == '__main__':
    main()



import unittest
import os
from .support import TempdirManager
from wmc.assemble import Interface


class TestInterface(TempdirManager, unittest.TestCase):

    def test_create(self):
        dir = self.mkdtemp()
        self.assertRaises(Exception, Interface, dir)

        interface = Interface(dir, check=False)
        self.assertIsInstance(interface, Interface)
        interface = Interface(dir, create=True)
        self.assertIsInstance(interface, Interface)


if __name__ == '__main__':
    unittest.main()



"""Some constant default values"""

COMMON = {
    'prefix': 'video_',
    "intro": {
        "prompt": ">>> ",
        "wait": 1,
        "font": "univers",
        "cmds": [
            {
                "input": "cat intro.txt",
                "output": [
                    {"text": "\n\n\n\n"},
                    {"title": "Watch me coding"},
                    {"text": "\n\n\n"},
                ],
            },
            {
                "input": "python load_info.py",
                "output": [
                    {"progressbar": None},
                    {"text": "I code some crazy stuff...\n"},
                ],
            },
            {
                "input": "vcl video.mp4",
                "output": [
                    {"text": "[*] Load video"},
                    {"progressbar": None},
                ],
            },
        ],
    }
}

LINUX = {
    'size':  [0, 0, 1920, 1080],
    'record': {
        "input": {
            "filename": ":0.0+0,0",
            "f": "x11grab",
            "video_size": (1920, 1080),
        },
        "output": {
            "vcodec": "libx264",
            "preset": "ultrafast",
            "r": 30,
        },
        "setpts": "N/TB/30",
    },
    'intro-record': {
        "input": {
            "filename": ":0.0+0,0",
            "f": "x11grab",
            "video_size": (1920, 1080),
        },
        "output": {
            "vcodec": "libx264",
            "preset": "ultrafast",
            "r": 30,
        },
        "setpts": "N/TB/30",
    },
}

WIN = {
    'size': [-1928, -4, 1935, 1093],
    'record': {
        "input": {
            "filename": "desktop",
            "f": "gdigrab",
            "framerate": 1,
            "video_size": (1920, 1080),
            "offset_x": -1920,
            "show_region": 1,
        },
        "output": {
            "vcodec": "libx264",
            "preset": "ultrafast",
            "r": 30,
        },
        "setpts": "N/TB/30",
    },
    'intro-record': {
        "input": {
            "filename": "desktop",
            "f": "gdigrab",
            "framerate": 1,
            "video_size": (1920, 1080),
            "offset_x": -1920,
            "show_region": 1,
        },
        "output": {
            "vcodec": "libx264",
            "preset": "ultrafast",
            "r": 30,
        },
        "setpts": "N/TB/30",
    },
}



import unittest
import os
from .support import TempdirManager
from wmc.project import ProjectData


class TestProjectData(TempdirManager, unittest.TestCase):

    def test_basic(self):
        dir = self.mkdtemp()
        path = os.path.join(dir, 'test_data.json')
        data = ProjectData(path)
        data.create()
        for key in ['record', 'size', 'prefix']:
            self.assertIn(key, data)


if __name__ == '__main__':
    unittest.main()



import os
import sys
from setuptools import setup, find_packages
from setuptools.command.develop import develop
from setuptools.command.install import install
from subprocess import check_call


class CheckFfmpegDevelop(develop):
    """Post-installation for development mode."""
    def run(self):
        check_call("ffmpeg -version".split())
        develop.run(self)


class CheckFfmpegInstall(install):
    """Post-installation for installation mode."""
    def run(self):
        check_call("ffmpeg -version".split())
        install.run(self)


base_dir = os.path.dirname(__file__)

with open(os.path.join(base_dir, 'README.rst')) as readme:
    README = readme.read()

requirements = ['lying==0.0.1a1', 'ffmpeg-python']
if sys.platform.startswith('win'):
    requirements.append('pywin32')
elif sys.platform.startswith('linux'):
    requirements.append('Xlib')

setup(
    name='wmc',
    description='Watch me coding, a toolbox',
    url='https://github.com/axju/wmc',
    author='Axel Juraske',
    author_email='axel.juraske@short-report.de',
    license='MIT',
    long_description=README,
    classifiers=[
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],

    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    use_scm_version=True,
    install_requires=requirements,
    setup_requires=[
        'setuptools_scm',
    ],
    entry_points={
        'console_scripts': [
            'wmc=wmc.cli:main',
        ]
    },

    cmdclass={
        'develop': CheckFfmpegDevelop,
        'install': CheckFfmpegInstall,
    },
)
